@@ -13,34 +13,22 @@
 import datetime
 import debtcollector
 import functools
+import io
 import itertools
 import logging
 import logging.config
 import logging.handlers
+import re
 import socket
 import sys
 import traceback
 
 from dateutil import tz
-import six
-from six import moves
 
 from oslo_context import context as context_utils
 from oslo_serialization import jsonutils
 from oslo_utils import encodeutils
 
-if six.PY3:
-    from functools import reduce
-
-
-try:
-    # Test if to_primitive() has the fallback parameter added
-    # in oslo.serialization 2.21.1
-    jsonutils.to_primitive(1, fallback=repr)
-    _HAVE_JSONUTILS_FALLBACK = True
-except TypeError:
-    _HAVE_JSONUTILS_FALLBACK = False
-
 
 def _dictify_context(context):
     if getattr(context, 'get_logging_values', None):
@@ -93,10 +81,10 @@ def _update_record_with_context(record):
 def _ensure_unicode(msg):
     """Do our best to turn the input argument into a unicode object.
     """
-    if isinstance(msg, six.text_type):
+    if isinstance(msg, str):
         return msg
-    if not isinstance(msg, six.binary_type):
-        return six.text_type(msg)
+    if not isinstance(msg, bytes):
+        return str(msg)
     return encodeutils.safe_decode(
         msg,
         incoming='utf-8',
@@ -165,7 +153,7 @@ def _get_error_summary(record):
                 error_summary = error_summary.split('\n', 1)[0]
         except TypeError as type_err:
             # Work around https://bugs.python.org/issue28603
-            error_summary = "<exception with %s>" % six.text_type(type_err)
+            error_summary = "<exception with %s>" % str(type_err)
         finally:
             # Remove the local reference to the exception and
             # traceback to avoid a memory leak through the frame
@@ -180,10 +168,24 @@ class _ReplaceFalseValue(dict):
         return dict.get(self, key, None) or '-'
 
 
+_MSG_KEY_REGEX = re.compile(r'(%+)\((\w+)\)')
+
+
+def _json_dumps_with_fallback(obj):
+    # Bug #1593641: If an object cannot be serialized to JSON, convert
+    # it using repr() to prevent serialization errors. Using repr() is
+    # not ideal, but serialization errors are unexpected on logs,
+    # especially when the code using logs is not aware that the
+    # JSONFormatter will be used.
+    convert = functools.partial(jsonutils.to_primitive, fallback=repr)
+    return jsonutils.dumps(obj, default=convert)
+
+
 class JSONFormatter(logging.Formatter):
-    def __init__(self, fmt=None, datefmt=None):
-        # NOTE(jkoelker) we ignore the fmt argument, but its still there
-        #                since logging.config.fileConfig passes it.
+    def __init__(self, fmt=None, datefmt=None, style='%'):
+        # NOTE(sfinucan) we ignore the fmt and style arguments, but they're
+        # still there since logging.config.fileConfig passes the former in
+        # Python < 3.2 and both in Python >= 3.2
         self.datefmt = datefmt
         try:
             self.hostname = socket.gethostname()
@@ -195,21 +197,32 @@ class JSONFormatter(logging.Formatter):
             lines = traceback.format_exception(*ei)
         except TypeError as type_error:
             # Work around https://bugs.python.org/issue28603
-            msg = six.text_type(type_error)
+            msg = str(type_error)
             lines = ['<Unprintable exception due to %s>\n' % msg]
         if strip_newlines:
-            lines = [moves.filter(
+            lines = [filter(
                 lambda x: x,
                 line.rstrip().splitlines()) for line in lines]
             lines = list(itertools.chain(*lines))
         return lines
 
     def format(self, record):
+        args = record.args
+        if isinstance(args, dict):
+            msg_keys = _MSG_KEY_REGEX.findall(record.msg)
+            # NOTE(bnemec): The logic around skipping escaped placeholders is
+            # tricky and error-prone to include in the regex.  Much easier to
+            # just grab them all and filter after the fact.
+            msg_keys = [m[1] for m in msg_keys if len(m[0]) == 1]
+            # If no named keys were found, then the entire dict must have been
+            # the value to be formatted.  Don't filter anything.
+            if msg_keys:
+                args = {k: v for k, v in args.items() if k in msg_keys}
         message = {'message': record.getMessage(),
                    'asctime': self.formatTime(record, self.datefmt),
                    'name': record.name,
                    'msg': record.msg,
-                   'args': record.args,
+                   'args': args,
                    'levelname': record.levelname,
                    'levelno': record.levelno,
                    'pathname': record.pathname,
@@ -255,16 +268,7 @@ class JSONFormatter(logging.Formatter):
         if record.exc_info:
             message['traceback'] = self.formatException(record.exc_info)
 
-        if _HAVE_JSONUTILS_FALLBACK:
-            # Bug #1593641: If an object cannot be serialized to JSON, convert
-            # it using repr() to prevent serialization errors. Using repr() is
-            # not ideal, but serialization errors are unexpected on logs,
-            # especially when the code using logs is not aware that the
-            # JSONFormatter will be used.
-            convert = functools.partial(jsonutils.to_primitive, fallback=repr)
-            return jsonutils.dumps(message, default=convert)
-        else:
-            return jsonutils.dumps(message)
+        return _json_dumps_with_fallback(message)
 
 
 class FluentFormatter(logging.Formatter):
@@ -277,25 +281,27 @@ class FluentFormatter(logging.Formatter):
     .. versionadded:: 3.17
     """
 
-    def __init__(self, fmt=None, datefmt=None):
-        # NOTE(masaki) we ignore the fmt argument because of the same reason
-        #              with JSONFormatter.
+    def __init__(self, fmt=None, datefmt=None, style='%s'):
+        # NOTE(sfinucan) we ignore the fmt and style arguments for the same
+        # reason as JSONFormatter.
         self.datefmt = datefmt
         try:
             self.hostname = socket.gethostname()
         except socket.error:
             self.hostname = None
+        self.cmdline = " ".join(sys.argv)
 
     def formatException(self, exc_info, strip_newlines=True):
         try:
             lines = traceback.format_exception(*exc_info)
         except TypeError as type_error:
             # Work around https://bugs.python.org/issue28603
-            msg = six.text_type(type_error)
+            msg = str(type_error)
             lines = ['<Unprintable exception due to %s>\n' % msg]
         if strip_newlines:
-            lines = reduce(lambda a, line: a + line.rstrip().splitlines(),
-                           lines, [])
+            lines = functools.reduce(lambda a,
+                                     line: a + line.rstrip().splitlines(),
+                                     lines, [])
         return lines
 
     def format(self, record):
@@ -308,6 +314,7 @@ class FluentFormatter(logging.Formatter):
                    'module': record.module,
                    'funcname': record.funcName,
                    'process_name': record.processName,
+                   'cmdline': self.cmdline,
                    'hostname': self.hostname,
                    'traceback': None,
                    'error_summary': _get_error_summary(record)}
@@ -334,6 +341,11 @@ class FluentFormatter(logging.Formatter):
         else:
             message['context'] = {}
         extra.pop('context', None)
+        # NOTE(vdrok): try to dump complex objects
+        primitive_types = (str, int, bool, type(None), float, list, dict)
+        for key, value in extra.items():
+            if not isinstance(value, primitive_types):
+                extra[key] = _json_dumps_with_fallback(value)
         message['extra'] = extra
 
         if record.exc_info:
@@ -383,20 +395,6 @@ class ContextFormatter(logging.Formatter):
 
     def format(self, record):
         """Uses contextstring if request_id is set, otherwise default."""
-        if six.PY2:
-            should_use_unicode = True
-            args = (record.args.values() if isinstance(record.args, dict)
-                    else record.args)
-            for arg in args or []:
-                try:
-                    six.text_type(arg)
-                except UnicodeDecodeError:
-                    should_use_unicode = False
-                    break
-            if (not isinstance(record.msg, six.text_type)
-                    and should_use_unicode):
-                record.msg = _ensure_unicode(record.msg)
-
         # store project info
         record.project = self.project
         record.version = self.version
@@ -507,16 +505,16 @@ class ContextFormatter(logging.Formatter):
                 return logging.Formatter.formatException(self, exc_info)
             except TypeError as type_error:
                 # Work around https://bugs.python.org/issue28603
-                msg = six.text_type(type_error)
+                msg = str(type_error)
                 return '<Unprintable exception due to %s>\n' % msg
 
-        stringbuffer = moves.StringIO()
+        stringbuffer = io.StringIO()
         try:
             traceback.print_exception(exc_info[0], exc_info[1], exc_info[2],
                                       None, stringbuffer)
         except TypeError as type_error:
             # Work around https://bugs.python.org/issue28603
-            msg = six.text_type(type_error)
+            msg = str(type_error)
             stringbuffer.write('<Unprintable exception due to %s>\n' % msg)
 
         lines = stringbuffer.getvalue().split('\n')
