@@ -16,7 +16,6 @@ import logging
 import logging.config
 import logging.handlers
 import os
-import six
 
 try:
     from systemd import journal
@@ -26,7 +25,6 @@ try:
     import syslog
 except ImportError:
     syslog = None
-from oslo_utils import encodeutils
 
 
 NullHandler = logging.NullHandler
@@ -71,29 +69,6 @@ class OSSysLogHandler(logging.Handler):
     def emit(self, record):
         priority = SYSLOG_MAP.get(record.levelname, 7)
         message = self.format(record)
-
-        # NOTE(gangila): In python2, the syslog function takes in 's' as
-        # the format argument, which means it either accepts python string
-        # (str = 'a') or unicode strings (str = u'a'), the PyArg_ParseTuple
-        # then if needed converts the unicode objects to C strings using
-        # the *default encoding*. This default encoding is 'ascii' in case
-        # of python2 while it has been changed to 'utf-8' in case of
-        # python3. What this leads to is when we supply a syslog message
-        # like:
-        # >>> syslog.syslog(syslog.LOG_DEBUG, u"François Deppierraz")
-        # In case of python2 the above fails with TypeError: [priority,]
-        # message string. Because python2 doesn't explicitly encode as
-        # 'utf-8' and use the system default encoding ascii, which raises
-        # a UnicodeEncodeError (UnicodeEncodeError: 'ascii' codec can't
-        # encode character u'\xe7' in position 4: ordinal not in
-        # range(128)), and hence the error message that's set in the code
-        # (TypeError: [priority,] message string) gets shown to the user.
-        # However, this in the case of Python3, where the system default
-        # encoding is 'utf-8' works without any issues. Therefore, we need
-        # to safe_encode in case of python2 and not in case of Python3.
-        if six.PY2:
-            message = encodeutils.safe_encode(self.format(record))
-
         syslog.syslog(priority, message)
 
 
@@ -131,11 +106,16 @@ class OSJournalHandler(logging.Handler):
             'PRIORITY': priority
         }
 
-        if record.exc_text:
-            extras['EXCEPTION_TEXT'] = record.exc_text
-
         if record.exc_info:
-            extras['EXCEPTION_INFO'] = record.exc_info
+            # Cache the traceback text to avoid converting it multiple times
+            # (it's constant anyway)
+            if not record.exc_text:
+                record.exc_text = self.formatter.formatException(
+                    record.exc_info)
+        if record.exc_text:
+            extras['EXCEPTION_INFO'] = record.exc_text
+            # Leave EXCEPTION_TEXT for backward compatibility
+            extras['EXCEPTION_TEXT'] = record.exc_text
 
         for field in self.custom_fields:
             value = record.__dict__.get(field)
@@ -146,6 +126,12 @@ class OSJournalHandler(logging.Handler):
 
 
 class ColorHandler(logging.StreamHandler):
+    """Log handler that sets the 'color' key based on the level
+
+    To use, include a '%(color)s' entry in the logging_context_format_string.
+    There is also a '%(reset_color)s' key that can be used to manually reset
+    the color within a log line.
+    """
     LEVEL_COLORS = {
         _TRACE: '\033[00;35m',  # MAGENTA
         logging.DEBUG: '\033[00;32m',  # GREEN
@@ -158,4 +144,5 @@ class ColorHandler(logging.StreamHandler):
 
     def format(self, record):
         record.color = self.LEVEL_COLORS[record.levelno]
-        return logging.StreamHandler.format(self, record)
+        record.reset_color = '\033[00m'
+        return logging.StreamHandler.format(self, record) + record.reset_color
