@@ -27,6 +27,7 @@ It also allows setting of formatting information through conf.
 
 """
 
+import configparser
 import logging
 import logging.config
 import logging.handlers
@@ -40,8 +41,7 @@ except ImportError:
 
 from oslo_config import cfg
 from oslo_utils import importutils
-import six
-from six import moves
+from oslo_utils import units
 
 from oslo_log._i18n import _
 from oslo_log import _options
@@ -60,6 +60,15 @@ TRACE = handlers._TRACE
 
 logging.addLevelName(TRACE, 'TRACE')
 
+LOG_ROTATE_INTERVAL_MAPPING = {
+    'seconds': 's',
+    'minutes': 'm',
+    'hours': 'h',
+    'days': 'd',
+    'weekday': 'w',
+    'midnight': 'midnight'
+}
+
 
 def _get_log_file_path(conf, binary=None):
     logfile = conf.log_file
@@ -216,14 +225,14 @@ def _load_log_config(log_config_append):
             # Reset all existing loggers before reloading config as fileConfig
             # does not reset non-child loggers.
             for logger in _iter_loggers():
-                logger.level = logging.NOTSET
+                logger.setLevel(logging.NOTSET)
                 logger.handlers = []
                 logger.propagate = 1
             logging.config.fileConfig(log_config_append,
                                       disable_existing_loggers=False)
             _load_log_config.old_time = new_time
-    except (moves.configparser.Error, KeyError, os.error) as exc:
-        raise LogConfigError(log_config_append, six.text_type(exc))
+    except (configparser.Error, KeyError, os.error) as exc:
+        raise LogConfigError(log_config_append, str(exc))
 
 
 def _mutate_hook(conf, fresh):
@@ -344,13 +353,33 @@ def _setup_logging_from_conf(conf, project, version):
 
     logpath = _get_log_file_path(conf)
     if logpath:
+        # On Windows, in-use files cannot be moved or deleted.
         if conf.watch_log_file and platform.system() == 'Linux':
             from oslo_log import watchers
             file_handler = watchers.FastWatchedFileHandler
+            filelog = file_handler(logpath)
+        elif conf.log_rotation_type.lower() == "interval":
+            file_handler = logging.handlers.TimedRotatingFileHandler
+            when = conf.log_rotate_interval_type.lower()
+            interval_type = LOG_ROTATE_INTERVAL_MAPPING[when]
+            # When weekday is configured, "when" has to be a value between
+            # 'w0'-'w6' (w0 for Monday, w1 for Tuesday, and so on)'
+            if interval_type == 'w':
+                interval_type = interval_type + str(conf.log_rotate_interval)
+            filelog = file_handler(logpath,
+                                   when=interval_type,
+                                   interval=conf.log_rotate_interval,
+                                   backupCount=conf.max_logfile_count)
+        elif conf.log_rotation_type.lower() == "size":
+            file_handler = logging.handlers.RotatingFileHandler
+            maxBytes = conf.max_logfile_size_mb * units.Mi
+            filelog = file_handler(logpath,
+                                   maxBytes=maxBytes,
+                                   backupCount=conf.max_logfile_count)
         else:
             file_handler = logging.handlers.WatchedFileHandler
+            filelog = file_handler(logpath)
 
-        filelog = file_handler(logpath)
         log_root.addHandler(filelog)
 
     if conf.use_stderr:
@@ -361,6 +390,14 @@ def _setup_logging_from_conf(conf, project, version):
         journal = handlers.OSJournalHandler()
         log_root.addHandler(journal)
 
+    if conf.use_eventlog:
+        if platform.system() == 'Windows':
+            eventlog = logging.handlers.NTEventLogHandler(project)
+            log_root.addHandler(eventlog)
+        else:
+            raise RuntimeError(_("Windows Event Log is not available on this "
+                                 "platform."))
+
     # if None of the above are True, then fall back to standard out
     if not logpath and not conf.use_stderr and not conf.use_journal:
         # pass sys.stdout as a positional argument
@@ -417,6 +454,7 @@ def _setup_logging_from_conf(conf, project, version):
                                   conf.rate_limit_interval,
                                   conf.rate_limit_except)
 
+
 _loggers = {}
 
 
